%%% Assemble stiffness matrix A and right hand side vector f

% Initialize A and f
A	= zeros(NoDofs, NoDofs);
f	= zeros(NoDofs, 1) ;

% Gauss quadrature points and weights
[P, W]     = GaussQuadrature(NoGaussPts, 2);

for e = 1:NoElem

	% Element interval (xi_i, xi_(i+1))
    Xi_e    = ElemRange(e,1:2);                                             
    Eta_e   = ElemRange(e,3:4);

	% Connectivities on element e and matrix B
    Conn    = ElemConn(e,:);  
	ConnB	= [Conn Conn+NoCtrlPts];
	
	nn	= length(Conn);
	B	= zeros(3, 2*nn);
	pts = controlPts(Conn,:); 
	R	= zeros(2,2*nn);
	
	for g = 1:size(W,1)

        xi_hat	= P(g,1);
        eta_hat = P(g,2); 
        w		= W(g);
		
		%Coordinates in parametric space:
        Xi  = 0.5 * ((Xi_e(2) - Xi_e(1))*xi_hat ...
                     + (Xi_e(2)  + Xi_e(1)));
        Eta = 0.5 * ((Eta_e(2) - Eta_e(1))*eta_hat ...
                     +(Eta_e(2) + Eta_e(1)));
		
        %Jacobian unit square - parametric:
        J2  = 0.25  * ( Xi_e(2) - Xi_e(1) )*( Eta_e(2) - Eta_e(1) );
        
		%Basis functions and derivatives
		[N, dNdxi, dNdeta] = GetBasisAndDerivatives(Xi, Eta, p, q, ...
													KnotVecX, KnotVecY);
		
		%Jacobian parametric - physical:
        jacob       = controlPts(Conn,:)'*[dNdxi' dNdeta'];
        J1          = det(jacob);
        invJacob    = inv(jacob);
		
		%Use the derivative wrt x and y instead of xi and eta:
		dNdx        = [dNdxi' dNdeta'] * invJacob;	
		
		%B-matrix:
		B(1,1:nn)       = dNdx(:,1)';
        B(2,nn+1:2*nn)  = dNdx(:,2)';
        B(3,1:nn)       = dNdx(:,2)';
        B(3,nn+1:2*nn)  = dNdx(:,1)';

		
		R(1,1:length(Conn)) = N';
		R(2,length(Conn)+1:end) = N';
		
		Dmat = ElasticityMatrix(Xi, Eta, Youngs, poissons);
		
		A(ConnB, ConnB) = A(ConnB, ConnB) + B'*Dmat*B*J1*J2*w;
		x = N*controlPts(Conn,1); y = N*controlPts(Conn,2);
		f(ConnB) = f(ConnB) + (func(x,y) * R * J1 * J2 * w)';
		

	end %g
end %e
