%%% Assemble stiffness matrix A and right hand side vector f

% Initialize A and f
A	= zeros(NoCtrlPts, NoCtrlPts);
f	= zeros(NoCtrlPts, 1) ;
exx	= zeros(NoElem, 1) ;

% Gauss quadrature points and weights
[P, W]     = GaussQuadrature(NoGaussPts, 2);


for e = 1:NoElem

	% Element interval (xi_i, xi_(i+1))
    Xi_e    = ElemRange(e,1:2);                                             
    Eta_e   = ElemRange(e,3:4);

	% Connectivities on element e
    Conn    = ElemConn(e,:);                                                
    
	for g = 1:size(W,1)

        xi_hat	= P(g,1);
        eta_hat = P(g,2); 
        w		= W(g);
		
		%Coordinates in parametric space:
        Xi  = 0.5 * ((Xi_e(2) - Xi_e(1))*xi_hat ...
                     + (Xi_e(2)  + Xi_e(1)));
        Eta = 0.5 * ((Eta_e(2) - Eta_e(1))*eta_hat ...
                     +(Eta_e(2) + Eta_e(1)));
		
        %Jacobian unit square - parametric:
        J2  = 0.25  * ( Xi_e(2) - Xi_e(1) )*( Eta_e(2) - Eta_e(1) );
        
		%Basis functions and derivatives
		[N, dNdxi, dNdeta] = GetBasisAndDerivatives(Xi, Eta, p, q, ...
													KnotVecX, KnotVecY);
		
		%Jacobian parametric - physical:
        jacob       = controlPts(Conn,:)'*[dNdxi' dNdeta'];
        J1          = det(jacob);
        invJacob    = inv(jacob);
		
		%Use the derivative wrt x and y instead of xi and eta
		dNdx        = [dNdxi' dNdeta'] * invJacob;							
		
		
		Alpha = alpha(Xi,Eta); 
		A(Conn, Conn) = A(Conn, Conn) + dNdx * Alpha * dNdx' * J1 * J2 * w;  
		x = N*controlPts(Conn,1); y = N*controlPts(Conn,2);
		f(Conn) = f(Conn) + (func(x,y) * N' * J1 * J2 * w);
	end %g
end %e
