
% Find basis functions at boundary
rightNodes	= find(controlPts(:,1) == endX)';
topNodes	= find(controlPts(:,2) == endY)';
leftNodes	= find(controlPts(:,1) == startX)';
bottomNodes	= find(controlPts(:,2) == startY)';	

% Solves least squares problem on boundary

	NoP = 10;%length(KnotVecX)*(p + 1)*10;
	
	NobottomNodes	= length(bottomNodes);
	NoleftNodes		= length(leftNodes);
	NorightNodes	= length(rightNodes);
	NotopNodes		= length(topNodes);

	% Used for local element indexing
	bottomElement	= zeros(NoElemX, p+1);
	leftElement		= zeros(NoElemY, q+1);
	rightElement	= zeros(NoElemY, q+1);
	topElement		= zeros(NoElemX, p+1);

	% Used to keep track on the active basis functions and
	% control points on each element boundary
	bottomEdgeMesh	= zeros(NoElemX, p+1);
	leftEdgeMesh	= zeros(NoElemY, q+1);
	rightEdgeMesh	= zeros(NoElemY, q+1);
	topEdgeMesh		= zeros(NoElemX, p+1);
	
	for i = 1:NoElemX
		bottomEdgeMesh(i,:) = bottomNodes(i:i+p);
		topEdgeMesh(i,:)	= topNodes(i:i+p);
		
		bottomElement(i,:)	= (i:i+p);
		topElement(i,:)		= (i:i+p);
	end	%i
	
	
	for i = 1:NoElemY
		leftEdgeMesh(i,:)	= leftNodes(i:i+q);
		rightEdgeMesh(i,:)	= rightNodes(i:i+q);
		
		leftElement(i,:)	= (i:i+q);
		rightElement(i,:)	= (i:i+q);
	end %i

	B = zeros(NoCtrlPts,1);
	C = zeros(NoCtrlPts, NoCtrlPts);
	
	bottomC	= zeros(NobottomNodes, NobottomNodes);
	leftC	= zeros(NoleftNodes, NoleftNodes);
	rightC	= zeros(NorightNodes, NorightNodes);
	topC	= zeros(NotopNodes, NotopNodes);

	bottomB = zeros(NobottomNodes,1);
	leftB	= zeros(NoleftNodes,1);
	rightB	= zeros(NorightNodes,1);
	topB	= zeros(NotopNodes,1);

%%% BOTTOM EDGE %%%

	for e = 1:NoElemX
		sctr = bottomEdgeMesh(e,:);
		sctrC = bottomElement(e,:);
		Xi_e = ElemRangeX(e,:);
		xiC = linspace(Xi_e(1), Xi_e(2), NoP);
		
		N = GetBasisBoundary(KnotVecX,p,xiC);
		bottomC(sctrC,sctrC) = bottomC(sctrC,sctrC) + N'*N;
		x = N*controlPts(sctr,1); y = N*controlPts(sctr,2);
		ff = BottomBoundary(x,y);
		bottomB(sctrC) = bottomB(sctrC) + N'*ff;

	end %e
	
	C(bottomNodes,bottomNodes) =  C(bottomNodes,bottomNodes) + bottomC;
	B(bottomNodes) = B(bottomNodes) +  bottomB;
	fBottom = bottomC\bottomB;
	
%%% LEFT EDGE %%%

	for e = 1:NoElemY
		sctr = leftEdgeMesh(e,:);
		sctrC = leftElement(e,:);
		Eta_e = ElemRangeY(e,:);
		etaC = linspace(Eta_e(1), Eta_e(2), NoP);

		M = GetBasisBoundary(KnotVecY,q,etaC);
		leftC(sctrC,sctrC) = leftC(sctrC,sctrC) + M'*M;
		x = M*controlPts(sctr,1); y = M*controlPts(sctr,2);
		ff = LeftBoundary(x,y);
		leftB(sctrC) = leftB(sctrC) + M'*ff;
	end %e

	C(leftNodes,leftNodes) =  C(leftNodes,leftNodes) + leftC;
	B(leftNodes) = B(leftNodes) +  leftB;
	fLeft = leftC\leftB;

%%% RIGHT EDGE %%%
	
	for e = 1:NoElemY
		sctr = rightEdgeMesh(e,:);
		sctrC = rightElement(e,:);
		Eta_e = ElemRangeY(e,:);
		etaC = linspace(Eta_e(1), Eta_e(2), NoP);
		
		M = GetBasisBoundary(KnotVecY,q,etaC);
		rightC(sctrC,sctrC) = rightC(sctrC,sctrC) + M'*M;
		x = M*controlPts(sctr,1); y = M*controlPts(sctr,2);
		ff = RightBoundary(x,y);
		rightB(sctrC) = rightB(sctrC) + M'*ff;
	end %e

 	C(rightNodes,rightNodes) =  C(rightNodes,rightNodes) + rightC;
 	B(rightNodes) = B(rightNodes) +  rightB;
	fRight = rightC\rightB;

%%% TOP EDGE %%%

	for e = 1:NoElemX
		sctr = topEdgeMesh(e,:);
		sctrC = topElement(e,:);
		Xi_e = ElemRangeX(e,:);
		xiC = linspace(Xi_e(1), Xi_e(2), NoP);
		N = GetBasisBoundary(KnotVecX,p,xiC);
		topC(sctrC,sctrC) = topC(sctrC,sctrC) + N'*N;
		x = N*controlPts(sctr,1); y = N*controlPts(sctr,2);
		ff = TopBoundary(x,y);
		topB(sctrC) = topB(sctrC) + N'*ff;
	end %e
	
	C(topNodes,topNodes) =  C(topNodes,topNodes) + topC;
	B(topNodes) = B(topNodes) +  topB;
	fTop = topC\topB;

%%%% SOLVING LSM %%%%

	% Take out the rows and columns not belonging to the boundary
	Fix = [bottomNodes leftNodes rightNodes topNodes];
	FixedNodes = unique(Fix);
	for i = size(C,1):-1:1;
		if not(ismember(i,FixedNodes))
			C(i,:) = [];
			C(:,i) = [];
			B(i) = [];
		end %if
	end %if

%%%%APPLY TO BOUNDARY %%%%%

    Fixed = (C\B)';
	f				= f - A(:,FixedNodes)*Fixed';
	f(FixedNodes)	= Fixed;
	A(FixedNodes,:) = 0;
	A(:,FixedNodes) = 0;

	for i = 1:length(FixedNodes)
		A(FixedNodes(i), FixedNodes(i)) = 1; 
	end %i



